import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;
import java.awt.image.*;
import java.io.*;

public class Tortue{
    private double xDebut = 300 , yDebut = 200;
    private double x, y;
    private double angle;
    private boolean crayonBaisse;
    private Color couleur;
    private double epaisseur;
    
    private FeuilleADessin figure;
    
    // Pour le remplissage
    private Color couleurRemplissage;
    private ArrayList sommetsRemplissage = new ArrayList();
    private boolean remplir = false;
    
    public Tortue(FeuilleADessin z){
        figure = z;
        init();
    }
    
    public void init(){
        couleur = Color.black;
        couleurRemplissage = Color.black;
        epaisseur = 1; // par dfaut
        x = xDebut;
        y = yDebut;
        angle = 0;
        crayonBaisse = true;
    }
    
    private void trace(double xN, double yN){ // Tracer  la vole, coordonnees fenetre
        if (remplir){
            if (sommetsRemplissage.size()== 0){
                sommetsRemplissage.add(new Point2D.Double(x, y));
            }
            sommetsRemplissage.add(new Point2D.Double(xN, yN));
        }
        if (crayonBaisse){
            Graphics2D g2D = figure.gTampon;
            g2D.setPaint(couleur);
            g2D.setStroke(new BasicStroke((float)epaisseur));
           
            g2D.draw(new Line2D.Double(x, y, xN, yN));
            figure.repaint();
        }
    }
    
    private double sin(double angle){
        return Math.sin(angle*Math.PI/180); // La conversion degre/radian existe dans classe Math
    }
    
    private double cos(double angle){
        return Math.cos(angle*Math.PI/180);
    }
    
    public void avance(double d){
        double xN, yN;
        xN = x + d * sin(angle);
        yN = y - d * cos(angle); // pour changer le sens des y
        trace(xN, yN);
        x = xN;
        y = yN;
    }
    
    public void av(double d){
        avance(d);
    }
    
    public void droite(double angle){
        this.angle += angle;
    }
    
    public void dr(double angle){
        droite(angle);
    }
    
    public void gauche(double angle){
        this.angle -= angle;
    }
    
    public void ga(double angle){
        this.angle -= angle;
    }
    
    public void maison(){
        trace(xDebut, yDebut);
        x = xDebut;
        y = yDebut;
    }
    
    public double getAngle(){
        return angle;
    }
    
    public void setAngle(double d){
        angle = d;
    }
    
    public double getX(){
        return x - xDebut;
    }
    
    public void setX(double x){
        this.x = x + xDebut;
    }
    
    public double getY(){
        return yDebut - y;
    }
    
    public void setY(double y){
        this.y = yDebut - y;
    }
    
    public void baisseCrayon(){
        crayonBaisse = true;
    }
    public void bC(){
        baisseCrayon();
    }
    
    public void leveCrayon(){
        crayonBaisse = false;
    }
    
    public void lC(){
        leveCrayon();
    }
    
    public void setPos(double x, double y){
        trace(x + xDebut, yDebut - y); //trace(x, y);
        this.x = x + xDebut;
        this.y = yDebut - y;
    }
    
    public void setColor(Color c){
        couleur = c;
    }
    
    public Color getColor(){
        return couleur;
    }
    
    public void setEpaisseur(double ep){
        epaisseur = ep;
    }
    
    public double getEpaisseur(){
        return epaisseur;
    }
    
    public double vers(double x, double y){
        return 90 - Math.atan2(y - getY() , x - getX()) * 180/ Math.PI;
    }
    
    public void setxDebut(double x){
        xDebut = x;
    }
    
    public void setyDebut(double y){
        yDebut = y;
    }
    
    public double getxDebut(){
        return xDebut;
    }
    
    public double getyDebut(){
        return yDebut;
    }
    
    // Transformer coordonnes cran en coordonnes tortue
    public double xEcranVersTortue(double u){
        return u - xDebut;
    }
    
    public double yEcranVersTortue(double v){
        return yDebut - v;
    }
    
    // Transformer coordonnes tortue en coordonnes cran
    public double xTortueVersEcran(double u){
        return u + xDebut;
    }
    
    public double yTortueVersEcran(double v){
        return yDebut - v;
    }
    
    public void ecrire(double u, double v, String texte){// Coordonnes tortue
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleur);
        // On revient aux coordonnes cran
        g2D.drawString(texte, (int)(u + xDebut), (int)(yDebut -v));
        figure.repaint();
    }
    
    public void tailleTexte(int n){
        if (n<=0){return;}
        Graphics2D g2D = figure.gTampon;
        Font ancienne = g2D.getFont();
        Font nouvelle = ancienne.deriveFont((float)n);
        g2D.setFont(nouvelle);
    }
    
    public int hauteurTexte(){
        Graphics2D g2D = figure.gTampon;
        FontMetrics metric = figure.gTampon.getFontMetrics(g2D.getFont());
        return metric.getHeight();
    }
    
    public int longueurTexte(String ch){
        if (ch == null){return 0;}
        Graphics2D g2D = figure.gTampon;
        FontMetrics metric = figure.gTampon.getFontMetrics(g2D.getFont());
        return metric.stringWidth(ch);
    }
    
    public Color getColorFill(){
        return couleurRemplissage;
    }
    
    public void setColorFill(Color c){
        couleurRemplissage = c;
    }
    
    public void setColorFill(int r, int g, int b){
        couleurRemplissage = new Color(r, g, b);
    }
    
    public void debutRemplir(){
        remplir = true;
        sommetsRemplissage.clear();
    }
    
    public void finRemplir(){
        remplir = false;
        GeneralPath polygone = new GeneralPath(GeneralPath.WIND_EVEN_ODD,
                                            sommetsRemplissage.size());
        Point2D.Double pt = (Point2D.Double) sommetsRemplissage.get(0);                                    
        polygone.moveTo((float) pt.x,(float) pt.y);                                    
        for (int i = 1; i < sommetsRemplissage.size(); i++){
            pt = (Point2D.Double) sommetsRemplissage.get(i);
            polygone.lineTo((float) pt.x,(float) pt.y);
        }
        polygone.closePath();
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleurRemplissage);
        g2D.fill(polygone);
        figure.repaint();
        sommetsRemplissage.clear();
    }
    
    // Graphiques sans tortue
    
    public void segment(double x1, double y1, double x2, double y2){// Coordonne tortue
        // passage en coordonnes fentre
        x1 += xDebut; x2 += xDebut;
        y1 = yDebut - y1; y2 = yDebut - y2;
        if (remplir){
            if (sommetsRemplissage.size()==0){
                sommetsRemplissage.add(new Point2D.Double(x1, y1));
            }
            sommetsRemplissage.add(new Point2D.Double(x2, y2));
        }
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleur);
        g2D.setStroke(new BasicStroke((float)epaisseur));
        g2D.draw(new Line2D.Double(x1, y1, x2, y2));
        figure.repaint();
    }
    
    public void rectangle(double x, double y, double base, double hauteur){
        x += xDebut; y = yDebut - y;
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleur);
        g2D.setStroke(new BasicStroke((float)epaisseur));
        g2D.draw(new Rectangle2D.Double(x, y, base, hauteur));
        figure.repaint(); 
    }
    
     public void ellipse(double x, double y, double base, double hauteur){
        x += xDebut; y = yDebut - y;
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleur);
        g2D.setStroke(new BasicStroke((float)epaisseur));
        g2D.draw(new Ellipse2D.Double(x, y, base, hauteur));
        figure.repaint(); 
    }
    
    public void cercle(double cx, double cy, double rayon){
        ellipse(cx - rayon, cy + rayon, 2*rayon, 2*rayon);
    }
    
    public void arc(double cx, double cy, double rayon, double angleDeb, double angleFin){
        cx += xDebut; cy = yDebut - cy; angleDeb = 90 -angleDeb; angleFin = 90 - angleFin;
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleur);
        g2D.setStroke(new BasicStroke((float)epaisseur));
        g2D.draw(new Arc2D.Double(cx - rayon, cy -rayon, 2*rayon, 2*rayon, angleDeb, angleFin, Arc2D.OPEN));
        figure.repaint();
    }
    
    public void rectanglePlein(double x, double y, double base, double hauteur){
        x += xDebut; y = yDebut - y;
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleurRemplissage);
        g2D.fill(new Rectangle2D.Double(x, y, base, hauteur));
        figure.repaint(); 
    }
    public void ellipsePleine(double x, double y, double base, double hauteur){
        x += xDebut; y = yDebut - y;
        Graphics2D g2D = figure.gTampon;
        g2D.setPaint(couleurRemplissage);
        g2D.fill(new Ellipse2D.Double(x, y, base, hauteur));
        figure.repaint(); 
    }
    
    public void disque(double cx, double cy, double rayon){
        ellipsePleine(cx - rayon, cy + rayon, 2*rayon, 2*rayon);
    }
/*
 * Ajouts mars 2009
 */ 
    
    public Color getRGB(double x, double y){
        return figure.getRGB((int)Math.round(x + xDebut), (int)Math.round(yDebut - y));
    }
    
    public void setRGB(double x, double y, Color c){
        figure.setRGB((int)Math.round(x + xDebut), (int)Math.round(yDebut - y), c);
    }
    
    public void setRGB(double x, double y){ 
        figure.setRGB((int)Math.round(x + xDebut), (int)Math.round(yDebut - y), couleur);
    }
    
    public void setRGB(double x, double y, int r, int g, int b){ 
        figure.setRGB((int)Math.round(x + xDebut), (int)Math.round(yDebut - y), r, g, b);
    }
    
/*
 * Ajouts juillet et novembre 2009
 */
//********** Ajouts pour traitement d'images *************
    
    public void afficherImage(BufferedImage img, double x, double y, double l, double h) {
        if (img == null){return;}
        x = Math.round(x + xDebut);
        x = Math.max(x, 0);
        x = Math.min(x, figure.getTailleImage());
        y = Math.round(yDebut-y);
        y = Math.max(y, 0);
        y = Math.min(y, figure.getTailleImage());
        Graphics2D g2 = figure.gTampon;
        g2.setPaint(Color.white);
        g2.drawImage(img, (int) x, (int) y, (int) Math.round(l), (int) Math.round(h), null);
    }
    
    public void sauverImage(String nomFichier, double x, double y, double l, double h){
        BufferedImage imag = figure.imgTampon;
        x = Math.round(x + xDebut);
        x = Math.max(x, 0);
        x = Math.min(x, figure.getTailleImage());
        y = Math.round(yDebut - y);
        y = Math.max(y, 0);
        y = Math.min(y, figure.getTailleImage());
        File fichier = new File(nomFichier+".png");
        try{
            BufferedImage img = imag.getSubimage((int)x, (int)y, (int)Math.round(l), (int)Math.round(h));
            javax.imageio.ImageIO.write(img, "png", fichier);
        }catch(IOException e){System.out.println("Problmes en enregistrant " + fichier);}
    }
    
    public BufferedImage copierImage(double x, double y, double l, double h){
        BufferedImage imag = figure.imgTampon;
        x = Math.round(x + xDebut);
        x = Math.max(x, 0);
        x = Math.min(x, figure.getTailleImage());
        y = Math.round(yDebut - y);
        y = Math.max(y, 0);
        y = Math.min(y, figure.getTailleImage());
        try{
            BufferedImage img = imag.getSubimage((int)x, (int)y, (int)Math.round(l), (int)Math.round(h));
            return img;
        }catch(Exception e){
            return null;
        }
    }
}
